@@ -42,7 +42,7 @@ void reconnect()
 {
 	char stringbuffer[WRITE_BUFFER_LENGTH + 1];
 #ifdef DEBUG
-        oer_debug(OER_DEBUG_FLOOD, "reconnect()\n");
+        oer_debug(OER_DEBUG_NOISE, "reconnect()\n");
 #endif
 	/* check if connected */
 	if(mystate->current_server == NULL) {
@@ -130,7 +130,7 @@ void reconnect()
 void establishconnection()
 {
 #ifdef DEBUG
-        oer_debug(OER_DEBUG_FLOOD, "establishconnection()\n");
+        oer_debug(OER_DEBUG_NOISE, "establishconnection()\n");
 #endif
 	/* loop until a fatal error */
 	while(!mystate->bailout) {
@@ -196,7 +196,7 @@ int serverconnection()
 	struct hostent *hptr;
 	struct sockaddr_in servaddr;
 #ifdef DEBUG
-        oer_debug(OER_DEBUG_FLOOD, "serverconnection()\n");
+        oer_debug(OER_DEBUG_NOISE, "serverconnection()\n");
 #endif
 	/* check if we are already connected */
 	if(mystate->current_server != NULL) {
@@ -322,7 +322,7 @@ int registerconnection()
 	char *ptr;
 	struct passwd *pwd;
 #ifdef DEBUG
-        oer_debug(OER_DEBUG_FLOOD, "registerconnection()\n");
+        oer_debug(OER_DEBUG_NOISE, "registerconnection()\n");
 #endif
 	/* check if connected */
 	if(mystate->current_server == NULL) {
@@ -416,7 +416,7 @@ int handleserverdata()
         size_t br;
         char *ptr;
 #ifdef DEBUG
-        oer_debug(OER_DEBUG_FLOOD, "handleserverdata()\n");
+        oer_debug(OER_DEBUG_NOISE, "handleserverdata()\n");
 #endif
 	i = 0;
 	cont = 1;
@@ -478,7 +478,7 @@ int processtimeds()
 	struct timed *t2;
 	char stringbuffer[WRITE_BUFFER_LENGTH + 1];
 #ifdef DEBUG
-        oer_debug(OER_DEBUG_FLOOD, "processtimeds()\n");
+        oer_debug(OER_DEBUG_NOISE, "processtimeds()\n");
 #endif
 	/* check if connected */
 	if(mystate->current_server == NULL) {
@@ -550,7 +550,7 @@ int processactions(struct timed *t)
 #ifdef DEBUG
 	/* prevent initial flooding of the console */
 	if((mystate->now - mystate->current_server->linkup) >= 10) {
-		oer_debug(OER_DEBUG_FLOOD, "processactions(\"%lu\")\n", t->at);
+		oer_debug(OER_DEBUG_NOISE, "processactions(\"%lu\")\n", t->at);
 		oer_debug(OER_DEBUG_INFO, "processactions->last action at %lu / delta %d / barrier %d\n", mystate->last_action.ts, (int) (mystate->now - mystate->last_action.ts), mystate->last_action.barrier);
 	}
 #endif
