@@ -210,6 +210,7 @@ enum OER_DEBUG_LEVELS {
 	OER_DEBUG_ERROR = 2,
 	OER_DEBUG_WARNING = 3,
 	OER_DEBUG_INFO = 4,
+	OER_DEBUG_NOISE = 8,
 	OER_DEBUG_FLOOD = 9
 };
 
@@ -337,8 +338,9 @@ enum OER_TIMED_PRIORITIES {
 #define OER_HELP "For a complete command reference go to http://oer.equnet.org/help.html"
 #define DEFAULT_NICKBK_MESSAGE "prohibited nick"
 #define DEFAULT_WORDBK_MESSAGE "word ban-kick triggered, have a nice life..."
+#define OER_SERVER_IS_STONED "stoned server"
 #define OER_REALNAME "http://oer.equnet.org/help.html"
-#define OER_COPYRIGHT1 "oer version 1.0-50, Copyright (C) 2000-2001 EQU <equ@equnet.org>"
+#define OER_COPYRIGHT1 "oer version 1.0-51, Copyright (C) 2000-2001 EQU <equ@equnet.org>"
 #define OER_COPYRIGHT2 "oer comes with ABSOLUTELY NO WARRANTY; for details see the\n\
 GNU General Public License (COPYING) provided with this distribution.\n\
 This is free software, and you are welcome to redistribute it under\n\
@@ -545,6 +547,7 @@ struct channel {
 	int synced;
 	int bantype;
 	int allhostsknown;
+	int whoquery;
 	time_t rejoin_at;
 	time_t last_quote;
 	time_t join_ts;
