@@ -296,7 +296,7 @@ int main(int argc, char *argv[])
 static void handlesignal(int signal)
 {
 #ifdef DEBUG
-        oer_debug(OER_DEBUG_FLOOD, "handlesignal(%d)\n", signal);
+        oer_debug(OER_DEBUG_NOISE, "handlesignal(%d)\n", signal);
 #endif
 	if(signal == SIGHUP) {
 #ifdef DEBUG
