@@ -55,6 +55,7 @@ void parseirc(char *serveroutput)
 	char outstring3[WRITE_BUFFER_LENGTH + 1];
 	char channel[CHANLEN + 1];
 	char mode[CHANLEN + 1];
+	char modeline[CHANLEN + 1];
 	char user[USERHOSTLEN + 1];
 	char host[USERHOSTLEN + 1];
 	char userhost[USERHOSTLEN + 1];
@@ -72,7 +73,7 @@ void parseirc(char *serveroutput)
 
 	striplf(serveroutput);
 #ifdef DEBUG
-        oer_debug(OER_DEBUG_FLOOD, "parseirc(\"%s\")\n", serveroutput);
+        oer_debug(OER_DEBUG_NOISE, "parseirc(\"%s\")\n", serveroutput);
 #endif
 	if(serveroutput[0] != ':') {
 		/* must be a command */
@@ -180,6 +181,18 @@ void parseirc(char *serveroutput)
 		/* this is the response to our ison query */
 		return;
 	}
+	if(!strcasecmp(command, "315")) {
+		/* end of WHO */
+		restpos = 0;
+		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
+			return;
+		}
+		if((this = getchptr(channel)) == NULL) {
+			return;
+		}
+		this->whoquery = 1;
+		return;
+	}
 	if(!strcasecmp(command, "332")) {
 		restpos = 0;
 		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
@@ -213,6 +226,43 @@ void parseirc(char *serveroutput)
 		/* returned when inviting someone */
 		return;
 	}
+	if(!strcasecmp(command, "352")) {
+		/* returned by the WHO query initiated in isjoined() */
+		restpos = 0;
+		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
+			return;
+		}
+		if((restpos = parse(restptr, restpos, " ", user, USERHOSTLEN, 0)) < 0) {
+			return;
+		}
+		if((restpos = parse(restptr, restpos, " ", host, USERHOSTLEN, 0)) < 0) {
+			return;
+		}
+		if((restpos = parse(restptr, restpos, " ", outstring, STRINGLEN, 0)) < 0) {
+			return;
+		}
+		if((restpos = parse(restptr, restpos, " ", nick, NICKLEN, 0)) < 0) {
+			return;
+		}
+		if((restpos = parse(restptr, restpos, " ", mode, CHANLEN, 0)) < 0) {
+			return;
+		}
+		if((this = getchptr(channel)) == NULL) {
+			return;
+		}
+		snprintf(userhost, USERHOSTLEN, "%s@%s", user, host);
+		setuserhost(nick, userhost);
+		if(index(mode, (int)'@')) {
+			changeuser(this, nick, -1, 1, -1, -1);
+		}
+		if(index(mode, (int)'*')) {
+			changeuser(this, nick, 1, -1, -1, -1);
+		}
+		if(index(mode, (int)'+')) {
+			changeuser(this, nick, -1, -1, 1, -1);
+		}
+		return;
+	}
 	if(!strcasecmp(command, "353")) {
 		restpos = 0;
 		if((restpos = parse(restptr, restpos, " ", mode, CHANLEN, 0)) < 0) {
@@ -812,8 +862,19 @@ void parseirc(char *serveroutput)
 					return;
 				}
 				restpos = nrestpos;
+				getnthmode(mode, i + 1, modeline);
+				if(!strcasecmp(modeline, "+k")) {
+					strncpy(this->key, target, CHANLEN);
+					snprintf(outstring, STRINGLEN, "%s channel key is now %s", this->name, this->key);
+					sendreply(this->name, 1, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
+				}
+				if(!strcasecmp(modeline, "-k")) {
+					memset(this->key, 0, CHANLEN);
+					snprintf(outstring, STRINGLEN, "deleted channel key from %s", this->name);
+					sendreply(this->name, 1, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
+				}
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->target is %s\n", target);
+				oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->target is %s\n", target);
 #endif
 				break;
 			case OER_NTHMODE_UNKNOWN:
@@ -822,12 +883,14 @@ void parseirc(char *serveroutput)
 			switch(modetype) {
 			case OER_NTHMODE_OP:
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_OP\n");
+				oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->OER_NTHMODE_OP\n");
 #endif
 				changeuser(this, target, -1, 1, -1, -1);
 				if(isme(target)) {
 					this->i_am_op = 1;
-					channelsync(this);
+					if(this->allhostsknown) {
+						channelsync(this);
+					}
 					continue;
 				}
 				if(!this->i_am_op) {
@@ -839,14 +902,14 @@ void parseirc(char *serveroutput)
 						continue;
 					}
 #ifdef DEBUG
-					oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->non-allowed op of nick %s on channel %s by %s!%s\n", target, channel, temp_nick, userhost);
+					oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->non-allowed op of nick %s on channel %s by %s!%s\n", target, channel, temp_nick, userhost);
 #endif
 					mmode_new(this, mystate->now, "-o", target);
 				}
 				break;
 			case OER_NTHMODE_DOP:
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_DOP\n");
+				oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->OER_NTHMODE_DOP\n");
 #endif
 				changeuser(this, target, -1, 0, -1, -1);
 				if(isme(target)) {
@@ -864,19 +927,19 @@ void parseirc(char *serveroutput)
 				break;
 			case OER_NTHMODE_VOICE:
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_VOICE\n");
+				oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->OER_NTHMODE_VOICE\n");
 #endif
 				changeuser(this, target, -1, -1, 1, -1);
 				break;
 			case OER_NTHMODE_DVOICE:
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_DVOICE\n");
+				oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->OER_NTHMODE_DVOICE\n");
 #endif
 				changeuser(this, target, -1, -1, 0, -1);
 				break;
 			case OER_NTHMODE_BAN:
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_BAN\n");
+				oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->OER_NTHMODE_BAN\n");
 #endif
 				snprintf(outstring, STRINGLEN, "%s!%s@%s", mystate->nick, mystate->user, mystate->host);
 				if(wild_match(target, outstring)) {
@@ -900,7 +963,7 @@ void parseirc(char *serveroutput)
 				break;
 			case OER_NTHMODE_UNBAN:
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_UNBAN\n");
+				oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->OER_NTHMODE_UNBAN\n");
 #endif
 				if(this->i_am_op && haschanflags(this, "e") && ispermban(this, target)) {
 					if(isq(temp_nick) || isservice(temp_nick)) {
@@ -917,7 +980,7 @@ void parseirc(char *serveroutput)
 				break;
 			case OER_NTHMODE_CHANMODE_WITHOUT_PARAMS:
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_CHANMODE_WITHOUT_PARAMS\n");
+				oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->OER_NTHMODE_CHANMODE_WITHOUT_PARAMS\n");
 #endif
 				if(!isme(temp_nick)) {
 					this->setchanmode = 1;
@@ -925,7 +988,7 @@ void parseirc(char *serveroutput)
 				break;
 			case OER_NTHMODE_CHANMODE_WITH_PARAMS:
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_FLOOD, "parseirc(MODE)->OER_NTHMODE_CHANMODE_WITH_PARAMS\n");
+				oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->OER_NTHMODE_CHANMODE_WITH_PARAMS\n");
 #endif
 				if(!isme(temp_nick)) {
 					this->setchanmode = 1;
@@ -2210,12 +2273,13 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			return;
 		}
                 proceed = (isopa(userhost)) ? 1 : 0;
-		if(haschanflags(this, "U") && isatleastop(this, nick, userhost)) {
+		if(haschanflags(this, "U")) {
+			if(isatleastop(this, nick, userhost)) {
+				proceed = 1;
+			}
+		} else if(isatleastopnow(this, nick, userhost)) {
 			proceed = 1;
 		}
-                if(isatleastopnow(this, nick, userhost)) {
-			proceed = 1;
-                }
 		/* check if message was sent to a channel and if it was
 		   is "s" flag set */
 		if(tochan && haschanflags(this, "s")) {
@@ -2234,12 +2298,13 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			return;
 		}
                 proceed = (isopa(userhost)) ? 1 : 0;
-		if(haschanflags(this, "U") && isatleastop(this, nick, userhost)) {
+		if(haschanflags(this, "U")) {
+			if(isatleastop(this, nick, userhost)) {
+				proceed = 1;
+			}
+		} else if(isatleastopnow(this, nick, userhost)) {
 			proceed = 1;
 		}
-                if(isatleastopnow(this, nick, userhost)) {
-			proceed = 1;
-                }
 		/* check if message was sent to a channel and if it was
 		   is "s" flag set */
 		if(tochan && haschanflags(this, "s")) {
@@ -2268,12 +2333,13 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			return;
 		}
                 proceed = (isopa(userhost)) ? 1 : 0;
-		if(haschanflags(this, "U") && isatleastop(this, nick, userhost)) {
+		if(haschanflags(this, "U")) {
+			if(isatleastop(this, nick, userhost)) {
+				proceed = 1;
+			}
+		} else if(isatleastopnow(this, nick, userhost)) {
 			proceed = 1;
 		}
-                if(isatleastopnow(this, nick, userhost)) {
-			proceed = 1;
-                }
 		/* check if message was sent to a channel and if it was
 		   is "s" flag set */
 		if(tochan && haschanflags(this, "s")) {
@@ -2292,12 +2358,13 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 			return;
 		}
                 proceed = (isopa(userhost)) ? 1 : 0;
-		if(haschanflags(this, "U") && isatleastop(this, nick, userhost)) {
+		if(haschanflags(this, "U")) {
+			if(isatleastop(this, nick, userhost)) {
+				proceed = 1;
+			}
+		} else if(isatleastopnow(this, nick, userhost)) {
 			proceed = 1;
 		}
-                if(isatleastopnow(this, nick, userhost)) {
-			proceed = 1;
-                }
 		/* check if message was sent to a channel and if it was
 		   is "s" flag set */
 		if(tochan && haschanflags(this, "s")) {
